/*
	Name:			amisearch 1.0

	Description:	search for a match on aminet

	Template:		KEY/A,HOST/K,PORT/K
					<KEY> [HOST] [PORT/N]
					- KEY		the key to look for
					- HOST		the host to connect
								to Default: ftp.uni-paderborn.de
					- PORT		the port to connect to
								Default: "service http" or 80

	Note:			on failure, error message are printed to stderr
					rather then stdout if possible
*/


parse arg host key .

/* open stderr for clean output */
if ~Open("STDERR","*","W") then STDERR = "STDOUT"

/* parse arguments */
parm.1.value = "ftp.uni-paderborn.de"
if ~ReadArgs("KEY/A,HOST/K,PORT/K") then do
	call PrintFault(IoErr(),"amisearch")
	exit
end

/* lookup host. We need host name in server request */
if ~GetHostByName("HOST",parm.1.value) then call err("can't lookup host")

/* the port to connect to */
if parm.2.flag then port = parm.2.value
else
	if GetServByName("SE",service,"tcp") then port = se.servport
	else port = 80

/* create a socket */
sock = socket("INET","STREAM","IP")
if sock<0 then call err("unable to create socket")

/* and connect */
sin.ADDRFAMILY = "INET"
sin.ADDRPORT   = port
sin.ADDRADDR   = host.hostAddrList.0
if connect(sock,"SIN")<0 then call err("connect error")

/* request. If we have to send it all */
CRLF = d2c(13)||d2c(10)
req = "GET /aminetbin/find?" || parm.0.value || " HTTP/1.0" || CRLF
req = req || "User-Agent: amisearch/1.0 (AmigaOS)" || CRLF
req = req || "Referer: http://" || host.hostName || "/find" || CRLF
req = req || "Accept-Language: it, en, *" || CRLF
req = req || "Host: " || host.hstoName || CRLF
req = req || "Accept: text/html;level=3" || CRLF
req = req || "Accept: text/html;version=3.0" || CRLF
req = req || "Accept: */*" || CRLF || CRLF
if send(sock,req)~=length(req) then call err("send error")

/* skip head */
len = 1
do while len>0 & C~=CRLF
	len = recvline(sock,"C",256)
end

if errno()~=0 then err("error receiving")

/* that's all */
len = 1
do while len>0
	len = recv(sock,"C",256)
	call WriteCH("STDOUT",c)
end
exit

/* print error to stderr */
err:
parse arg s
	call writeln("STDERR","amisearch:" s)
	exit
